
var AHTTPD_PREFIX = "/alterator";
var BRO_URL = "/bro";
var BRO_RESP_PREFIX = "receive_response (\"";
var BRO_RESP_POSTFIX = "\");\n";
var BRO_MAX_AGE_DEFAULT = 31536000; /* one year */

var BRO_GET_SESSION_COOKIE = /form_update_cookie\s*\(\s*"session"\s*,\s*"([^"]*)"/;
var BRO_GET_SESSION_TOKEN = /form_replace\s*\(\s*"\/([^"/]*)\/?"/;
var BRO_GET_MESSAGE = /form_message\s*\(\s*{\s*msg\s*:\s*"([^"]*)".*"([^"]*)?"\s*\);/;

function bro(data, callback) {
    if (!data) data = {};
    var xhr = new XMLHttpRequest ();
    xhr.open ("POST", AHTTPD_PREFIX +
                      ((data && data.session_token) ?
                           ("/" + data.session_token) :
                           "") +
                      BRO_URL + "?ajax=bro&msienocache=" + Math.random(),
              true);
    xhr.onreadystatechange = function () {
        if (this.readyState == XMLHttpRequest.DONE) {
            var resp = null;
            if (this.status == 200 /* OK */) {
                if (this.responseText) {
                    var respText = this.responseText;
                    if (respText.startsWith (BRO_RESP_PREFIX) &&
                        respText.endsWith (BRO_RESP_POSTFIX))
                    {
                        respText = respText.substr (BRO_RESP_PREFIX.length,
                                                    respText.length -
                                                      BRO_RESP_PREFIX.length -
                                                        BRO_RESP_POSTFIX.length);
                        try {
                            resp = JSON.parse (JSON.parse ('"' + respText + '"'));
                        } catch (e) {
                            resp = {
                                error : {
                                    message : (e instanceof SyntaxError ?
                                               e.message : "Sorry, bro, unknown JSON parse error"),
                                    responseText : respText,
                                    code : 1
                                }
                            };
                        }
                    }
                    else
                    {
                        resp = {
                            error : {
                                message : "Sorry, bro, response syntax error",
                                responseText : this.responseText,
                                code : 8
                            }
                        };
                    }
                }
            } else if (this.status == 403) {
                resp = {
                    error : {
                        message : "Sorry, bro, access denied. Have you set the session cookie?",
                        code : 9
                    }
                };
            } else {
                resp = {
                    error : {
                        message : "Sorry, bro, a really unexpected response",
                        responseText : this.responseText,
                        code : 10
                    }
                };
            }
            if (callback && callback instanceof Function) {
                callback ({
                    result : resp,
                    status : this.status,
                    statusText : this.statusText
                });
            }
        }
    };
    xhr.setRequestHeader ("Content-Type", "application/x-www-form-urlencoded");
    xhr.send ("data=" + (data ? encodeURIComponent (JSON.stringify (data)) : ""));
}

function _bro_handle_message (xhr, next) {
    var resp = null;
    var msg = BRO_GET_MESSAGE.exec (xhr.responseText);
    if (msg && msg[1]) {
        resp = {
            error : {
                message : msg[1],
                code : 11
            }
        };
        if (msg[2]) {
            var r_xhr = new XMLHttpRequest ();
            r_xhr.open ("GET", AHTTPD_PREFIX +
                               "/login?ajax=" + msg[2] +
                               "&msienocache=" + Math.random(),
                        true);
            r_xhr.onreadystatechange = function () {
                if (this.readyState == XMLHttpRequest.DONE) {
                    next (resp);
                }
            };
            r_xhr.send ();
        }
    }

    next (resp);
}

function bro_login (data, callback) {
    if (!data) data = {};
    var xhr = new XMLHttpRequest ();
    xhr.open ("POST", AHTTPD_PREFIX +
                      "/login?ajax=commit-login&continue=%2f&msienocache=" + Math.random(),
              true);
    xhr.onreadystatechange = function () {
        if (this.readyState == XMLHttpRequest.DONE) {
            var self = this;
            _bro_handle_message (self, function (resp) {
                if (!resp || !resp.error) {
                    var session_cookie = BRO_GET_SESSION_COOKIE.exec (self.responseText);
                    var session_token = BRO_GET_SESSION_TOKEN.exec (self.responseText);
                    if (session_cookie && session_cookie[1]) {
                        resp = {
                            session : {
                                session_cookie : session_cookie[1],
                                session_token : (session_token &&
                                                 session_token[1] ?
                                                 session_token[1] :
                                                 null)
                            }
                        };
                        if (data.max_age == null)
                            data.max_age = BRO_MAX_AGE_DEFAULT;
                        if (data.max_age) {
                            document.cookie = "session=" +
                                              resp.session.session_cookie +
                                              ";path=/;" +
                                              ";max-age=" + data.max_age +
                                              ";secure";
                        }
                    } else {
                        resp = {
                            error : {
                                message : "Sorry, bro, unexpected login response",
                                responseText : self.responseText,
                                code : 8
                            }
                        };
                    }
                }
                if (callback && callback instanceof Function) {
                    callback ({
                        result : resp,
                        status : self.status,
                        statusText : self.statusText
                    });
                }
            });
        }
    };
    xhr.setRequestHeader ("Content-Type", "application/x-www-form-urlencoded");
    xhr.send ("login=" + (data.login ? encodeURIComponent (data.login) : "") +
              "&password=" + (data.password ? encodeURIComponent (data.password) : "") +
              (data.locale ? "&locale=" + encodeURIComponent (data.locale) : ""));
}

function bro_logout (data, callback) {
    if (!data) data = {};
    var xhr = new XMLHttpRequest ();
    xhr.open ("POST", AHTTPD_PREFIX +
                      ((data && data.session_token) ?
                           ("/" + data.session_token) :
                           "") +
                      "/logout?ajax=logout&msienocache=" + Math.random(),
              true);
    xhr.onreadystatechange = function () {
        if (this.readyState == XMLHttpRequest.DONE) {
            var self = this;
            _bro_handle_message (self, function (resp) {
                if (!resp || !resp.error) {
                    var session_cookie = BRO_GET_SESSION_COOKIE.exec (self.responseText);
                    if (session_cookie && session_cookie[1] == "") {
                        resp = {
                            session : {
                                session_cookie : "",
                                session_token : ""
                            }
                        };
                        if (!data.keepcookie)
                            document.cookie = "session=";
                    } else {
                        resp = {
                            error : {
                                message : "Sorry, bro, unexpected logout response",
                                responseText : self.responseText,
                                code : 8
                            }
                        };
                    }
                }
                if (callback && callback instanceof Function) {
                    callback ({
                        result : resp,
                        status : self.status,
                        statusText : self.statusText
                    });
                }
            });
        }
    };
    xhr.setRequestHeader ("Content-Type", "application/x-www-form-urlencoded");
    xhr.send ((data.locale ? "locale=" + encodeURIComponent (data.locale) : ""));
}
