(define-module (ui bro ajax)
  :use-module (alterator ajax)
  :use-module (alterator plist)
  :use-module (alterator woo)
  :use-module (json)
  :export (bro))

(define (alist? lst)
  (and (pair? lst)
       (list? lst)
       (pair? (car lst))
       (not (pair? (caar lst)))))

(define (sanitize-list x)
  (cond
   ((and (not (list? x))
         (not (pair? x)))
    x)
   ((not (list? x))
    (sanitize-list (list x)))
   ((not (alist? x))
    `#(,@(map sanitize-list x)))
   (else
    (map (lambda (p)
           (if (pair? p)
             (cons (car p)
                   (sanitize-list (cdr p)))
             (cons "_"
                   (sanitize-list p))))
         x))))

(define (bro-handle)
  (let* ((data (form-value "data"))
         (resp (catch #t
                  (lambda ()
                    (let* ((data (json-string->scm data))
                           (woo-req (and (list? data)
                                         (assoc "woo" data)))
                           (name (and (pair? woo-req)
                                      (list? (cdr woo-req))
                                      (assoc "name" (cdr woo-req))))
                           (args (and (pair? woo-req)
                                      (list? (cdr woo-req))
                                      (assoc "args" (cdr woo-req)))))
                      (cond
                       ((not woo-req)
                        `((error . ((message . "Excuse me, bro, what should I do?")
                                    (code . 2)))))
                       ((not (pair? name))
                        `((error . ((message . "Sorry, bro, invalid woo call: you should specify a name")
                                    (code . 2)))))
                       (else
                        (let ((resp (apply woo-call (cdr name)
                                           (or (and (pair? args)
                                                    (list? (cdr args))
                                                    (alist->plist (cdr args)))
                                               '()))))
                          (cond
                           ((null? resp)
                            `((woo . ())))
                           ((= 1 (length resp))
                            `((woo . ((name . ,(caar resp))
                                      (result . ,(plist->alist (cdar resp)))))))
                           (else
                            `((woo . ,(map (lambda (x)
                                             `((name . ,(car x))
                                               (result . ,(plist->alist (cdr x)))))
                                           resp))))))))))
                  (lambda (key . args)
                    `((error . ,(case key
                                  ((json-invalid)
                                   (let* ((parser (and (pair? args)
                                                       (car args)))
                                          (parser-port (and parser
                                                            (json-parser-port parser))))
                                     `((message . "Sorry, bro, invalid input JSON")
                                       (line . ,(and  parser-port
                                                     (port-line parser-port)))
                                       (column . ,(and parser-port
                                                       (port-column parser-port)))
                                       (code . 3))))
                                  ((woo-error)
                                   `((message . ,(and (pair? args)
                                                      (car args)))
                                     (code . 4)))
                                  ((type-error)
                                   `((message . "Backend has returned a type error")
                                     (reasonlist . ,(car args))
                                     (code . 5)))
                                  ((wrong-type-arg wrong-number-of-args misc-error)
                                   `((message . ,(or (and (pair? args)
                                                          (pair? (cdr args))
                                                          (pair? (cddr args))
                                                          (if (and (pair? (cdddr args))
                                                                   (car args))
                                                            (format #f "Error evaluating ~A: ~A"
                                                                    (append (list (car args))
                                                                            (or (cadddr args) '()))
                                                                    (apply format #f (cadr args) (caddr args)))
                                                            (apply format #f (cadr args) (caddr args))))
                                                     "Sorry, bro, unknown Scheme syntax error"))
                                     (args . ,args)
                                     (code . 6)))
                                  (else
                                   `((message . "Sorry, bro, a really unknown error")
                                     (key . ,key)
                                     (args . ,args)
                                     (code . 7))))))))))
    (js "receive_response" (scm->json-string (sanitize-list resp)))))

(define (bro)
  (catch #t bro-handle
    (lambda (key . args)
      (js "receive_response"
          (format #f "{\"error\":{\"message\":\"Sorry, bro, your bro needs some medicine\",\"key\":~A,\"args\":~A,\"code\":100}}"
                  (->json (format #f "~A" key))
                  (->json (format #f "~S" args)))))))
